<?php

class APITest extends PHPUnit_Framework_TestCase
{
	protected $backupGlobalsBlacklist = array('master_ary', 'Supported_Object_Types');
	private $api_command;
	function setUp()
	{
		evil_global_setup();
		set_user('host_7', array('op5_admins'));

		# these tests rely on imported objects in test 1 to be available in test2, hence:
		require_once('include/import.inc.php');
		global $HOME, $IMPORTING;
		$cfgs = array($HOME.'testconfig.cfg');
		assert(import_config('phptap', $cfgs));
		$IMPORTING = false;

		$this->api_command = "OP5LIBCFG=${HOME}/yml /usr/bin/php api/monitor.php";
		$this->assertNotSame(false, file_put_contents('custom_config.php', '<?php $DEBUG = TRUE; $HOME = "'.realpath(getenv('WORK_DIR')).'/"; $Config["main"] = $HOME."nagios.cfg"; $Config["cgi"] = $HOME."cgi.cfg";'));
	}

	function tearDown()
	{
		$this->assertTrue(unlink('custom_config.php'));
		evil_global_teardown();
	}

	private function ensure_no_php_errors($output) {
		foreach ($output as $row) {
			$this->assertTrue(strpos($row, 'Fatal') === false && strpos($row, 'thrown') === false && strpos($row, 'Stack') === false, 'Nothing that looks like a PHP error in output');
		}
	}

	/**
	 * Runs a config export in a background process to see that the
	 * lock mechanism prevents a simultaneous config import.
	 * This test uses the same user in both processes.
	 */
	public function test_export_import_config() {
		set_user('superuser');

		// Assert that no one has a lock.
		$this->assertSame(false, nacoma_objlock::get_lock_owner('all', 0));

		// Start a config "export" (i.e. save DB to files) in the background.
		exec($this->api_command . ' -a save_config > /dev/null &');

		// Wait for lock to be set.
		$now = microtime(true);
		while (!$user = nacoma_objlock::get_lock_owner('all', 0)) {
			if (microtime(true) - $now > 5) {
				// Time out after 5 seconds. Fail test.
				$this->fail('get_lock_owner() check timed out. Could not get lock.');
			}
		}
		$this->assertSame('superuser', $user);

		// We try to start an "import" (i.e. files to DB) simultaneously.
		// This should cause an error.
		exec($this->api_command . ' -a undo_config', $output, $status);

		$expected_output = array(
			'The database is currently being exported by superuser',
			'This should merely take a few seconds, but while it\'s going on'
				. ', no other users may edit the database.'
		);
		$this->assertSame($expected_output, $output);
		$this->assertSame(0, $status);

		// Allow the export to finish, i.e. lock to be released.
		$now = microtime(true);
		while (nacoma_objlock::get_lock_owner('all', 0)) {
			if (microtime(true) - $now > 5) {
				// Time out after 5 seconds. Fail test.
				$this->fail('get_lock_owner check timed out. Couldn\'t get lock.');
			}
		}
	}

	/**
	* 	host objects, check that we are able to list hosts
	* 	Testing list_objects
	*/
	public function test_show_host_objects()
	{
		set_user('host_7', array('op5_admins'));
		exec($this->api_command.' -t host -a list_objects', $return, $status);
		$this->assertTrue(is_array($return), 'output');
		$this->assertEquals('0', $status, 'exit code 0');
	}

	/**
	* 	host objects, check that we are able to list hosts when supplying a username
	* 	Testing list_objects
	*/
	public function test_show_host_objects_with_username()
	{
		exec($this->api_command.' -t host -a list_objects -u test_user', $return, $status);
		$this->assertTrue(is_array($return), 'output');
		$this->assertEquals('0', $status, 'exit code 0');
	}

	/**
	 * *EVERYTHING* in here has interdependencies - GAH!
	 */
	public function test_host_stuff()
	{
		exec($this->api_command.' -t host -o host_name=API_TESTHOST -o alias=API_TESTHOST -o address=192.168.1.26 -o template=default-host-template -o contacts=contact1', $return, $status);
		$this->assertEmpty($return, "Got unexpected output in API call: ".var_export($return, true));
		$this->assertEquals('0', $status, 'exit code 0');

		$return = false;
		exec($this->api_command.' -t host -a list_objects', $return, $status);
		$this->assertTrue(in_array('API_TESTHOST', $return));
		$this->assertEquals('0', $status);

		$return = false;
		exec($this->api_command.' -t host -n API_TESTHOST -R host_name=API_TESTHOST_NEW -o contacts=contact1,contact2', $return, $status);
		$this->assertEmpty($return, "Expected no output, got ".var_export($return, true));
		$this->assertEquals('0', $status);

		$return = false;
		exec($this->api_command.' -t host -a list_objects', $return, $status);
		$this->assertContains('API_TESTHOST_NEW', $return);
		$this->assertEquals('0', $status);

		$return = false;
		exec($this->api_command.' -t service_template -a list_objects', $return, $status);
		$this->assertTrue(in_array('default-service', $return));
		$this->assertEquals('0', $status);

		$return = false;
		exec($this->api_command.' -t service -o template=default-service -o host_name=API_TESTHOST_NEW -o service_description=API_TEST_SERVICE -o check_command=check-host-alive', $return, $status);
		$this->assertEmpty($return);
		$this->assertEquals('0', $status);

		$return = false;
		exec($this->api_command.' -t service -a list_objects', $return, $status);
		$this->assertTrue(in_array('API_TESTHOST_NEW;API_TEST_SERVICE', $return), 'Test service should exist. Looked for API_TESTHOST_NEW;API_TEST_SERVICE in '.var_export($return, true));
		$this->assertEquals('0', $status);

		$return = false;
		exec($this->api_command.' -t service -a show_object -n "hostgroup_1;groupservice"', $return, $status);
		$this->assertEquals('0', $status, 'Showing hostgroup service should work');
		$this->assertTrue(strpos(implode("\n", $return), "hostgroup_name=hostgroup_1") !== false, 'We should see hostgroup_name in output, got: '. implode("\n", $return));

		$return = false;
		exec($this->api_command.' -t host -n API_TESTHOST_NEW -a delete', $return, $status);
		$this->assertEmpty($status);
		$this->assertEquals('0', $status);

		$return = false;
		exec($this->api_command.' -t service -a list_objects', $return, $status);
		$this->assertEquals(false, in_array('API_TEST_SERVICE', $return));
		$this->assertEquals('0', $status);

		exec($this->api_command.' -t host -a list_objects', $return, $status);
		$this->assertEquals(false, in_array('API_TESTHOST_NEW', $return));
		$this->assertEquals('0', $status);
	}

	/**
	* 	Add new contact
	*/
	public function test_add_contact()
	{
		exec($this->api_command.' -t contact -o contact_name=API_CONTACT -o alias=API_CONTACT -o email=noname@domain.com -o service_notification_period=none -o host_notification_period=none', $return, $status);
		$this->assertEmpty($return, "Contained ".var_export($return, true));
		$this->assertEquals('0', $status);

		$return = false;
		exec($this->api_command.' -t contact -a list_objects', $return, $status);
		$this->assertTrue(in_array('API_CONTACT', $return));
		$this->assertEquals('0', $status);

		$return = false;
		exec($this->api_command.' -t contact -n API_CONTACT -a delete', $return, $status);
		$this->assertEmpty($return);
		$this->assertEquals('0', $status);

		$return = false;
		exec($this->api_command.' -t contact -a list_objects', $return, $status);
		$this->assertNotContains('API_CONTACT', $return, "Got ". var_export($return, true));
		$this->assertEquals('0', $status);

		$return = false;
	    exec($this->api_command.' -t user -o Username=api_limited_user -o Password=api_limited_user -o "Access Levels=authorized_for_system_commands,authorized_for_configuration_information"', $return, $status);
	    $this->assertEmpty($return, 'User was created without errors');
	    $this->assertEquals('0', $status, 'User add was successful');

		$return = false;
		exec($this->api_command.' -t user -a list_objects', $return, $status);
		$this->assertTrue(in_array('api_limited_user', $return), "Expected api_limited_user to be in ".var_export($return, true));
		$this->assertEquals(0, $status);

		$return = false;
	    exec($this->api_command.' -t usergroup -a list_objects', $return, $status);
	    $this->assertTrue(in_array('user_api_limited_user', $return), 'Implicitly created usergroup is in group listing');
	    $this->assertEquals(0, $status);

		$return = false;
	    exec($this->api_command.' -t user -n api_limited_user -a delete', $return, $status);
	    $this->assertEmpty($return, 'User was deleted without errors');
	    $this->assertEquals('0', $status, 'User delete was successful');

		$return = false;
	    exec($this->api_command.' -t user -a list_objects', $return, $status);
	    $this->assertTrue(!in_array('api_limited_user', $return), 'User with permissions was deleted.');
	    $this->assertEquals(0, $status, 'User was deleted with no errors.');

		$return = false;
	    exec($this->api_command.' -t user -o Username=limited_user_with_groups -o Password=api_limited_user -o "groups=contact_limited"', $return, $status);
	    $this->assertEmpty($return, 'User was created without errors');
	    $this->assertEquals('0', $status, 'User add was successful');

		$return = false;
	    exec($this->api_command.' -t user -a list_objects', $return, $status);
	    $this->assertTrue(in_array('limited_user_with_groups', $return));
	    $this->assertEquals(0, $status);

		$return = false;
	    exec($this->api_command.' -t user -n limited_user_with_groups -a delete', $return, $status);
	    $this->assertEmpty($return, 'User was deleted without errors');
	    $this->assertEquals('0', $status, 'User delete was successful');

		$return = false;
	    exec($this->api_command.' -t user -a list_objects', $return, $status);
	    $this->assertTrue(!in_array('limited_user_with_groups', $return));
	    $this->assertEquals(0, $status);

		$return = false;
		exec($this->api_command.' -t usergroup -o name=test_group -o access_rights=host_view_all,host_edit_all,service_view_all,service_edit_all,system_commands,configuration_information,system_information', $return, $status);
		$this->assertEmpty($return, 'Group was created without errors');
		$this->assertEquals(0, $status, 'Group creation was successful');

		$return = false;
		exec($this->api_command.' -t usergroup -a show_object -n test_group', $return, $status);
		$this->assertTrue(strpos(implode("\n", $return), 'service_view_all') !== false, 'Expected permission flag was set on group.');

		$return = false;
		exec($this->api_command.' -t usergroup -n test_group -a delete', $return, $status);
		$this->assertEmpty($return, 'Group was deleted without errors');
		$this->assertEquals('0', $status, 'Group delete was successful');

		$return = false;
	    exec($this->api_command.' -t usergroup -a list_objects', $return, $status);
	    $this->assertTrue(!in_array('test_group', $return), 'Group was indeed deleted');
	    $this->assertEquals(0, $status, 'List groups was successful.');
	}

	/**
	*	Add new timeperiod API_TIMEPERIOD
	*/
	public function test_add_timeperiod()
	{
		exec($this->api_command.' -t timeperiod -o timeperiod_name=API_TIMEPERIOD -o alias=API_TIMEPERIOD', $return, $status);
		$this->assertEmpty($return);
		$this->assertEquals('0', $status);

		$return = false;
		exec($this->api_command.' -t timeperiod -a list_objects', $return, $status);
		$this->assertTrue(in_array('API_TIMEPERIOD', $return));
		$this->assertEquals('0', $status);

		$return = false;
		exec($this->api_command.' -t timeperiod -n API_TIMEPERIOD -a delete', $return, $status);
		$this->assertEquals(0, $status);

		$return = false;
		exec($this->api_command.' -t timeperiod -a list_objects', $return, $status);
		$this->assertEquals(false, in_array('API_TIMEPERIOD', $return));
		$this->assertEquals('0', $status);
		#$this->markTestSkipped('MARKING TEST AS SKIPPED');
	}

	public function test_add_servicedependency()
	{
		exec("/usr/bin/php api/monitor.php -t servicedependency -o 'dependent_service=host_10;CPU Load' -o 'service=host_17;PING'", $return, $status);
		$this->assertTrue(empty($return), 'Expected no output, but got '.implode("\n", $return));
		$this->assertEquals('0', $status, "Expected exit code 0, but was $status");

		$return = false;
		exec("/usr/bin/php api/monitor.php -t servicedependency -a list_objects", $return, $status);
		$this->assertEquals(0, $status);
		$found = false;
		foreach ($return as $line) {
			if (strpos($line, 'host service "host_10;CPU Load" depends on host service "host_17;PING" #') !== false) {
				$found = true;
			}
		}
		$this->assertTrue($found, "Couldn't find added servicedependency: wanted host service \"host_10;CPU Load\" depends on host service \"host_17;PING\" in ".implode("\n", $return));

		$return = false;
		exec("/usr/bin/php api/monitor.php -t servicedependency -a delete -i 1", $return, $status);
		$this->assertEquals(0, $status);
		$this->assertTrue(!in_array('host service "linux-server1;PING" depends on host service "service=printer1;PING" #1', $return));

		$return = false;
		exec("/usr/bin/php api/monitor.php -t servicedependency -a list_objects", $return, $status);
		$this->assertEquals(0, $status);
		$this->assertTrue(!in_array('host service "linux-server1;PING" depends on host service "service=printer1;PING" #1', $return));
	}

	public function test_invalid_rel()
	{
		exec("/usr/bin/php api/monitor.php -t host -o host_name=API_TEST_HOST -o contacts=API_INVALID_CONTACT", $return, $status);
		$this->assertEquals(1, $status, 'Contact not found should cause invalid exit code');
		$invalidness = false;
		foreach ($return as $row) {
			if (strpos($row, 'was not found') !== false) {
				$invalidness = true;
				break;
			}
		}
		$this->assertTrue($invalidness, 'Error message looks sane');
		$this->ensure_no_php_errors($return);
	}

	public function test_show_b0rked_obj_type()
	{
		$commands = array(
			$this->api_command.' -a show_object -n API_TEST_HOST',
			$this->api_command.' -a show_object -t host',
			$this->api_command.' -a show_object -t hosta -n API_TEST_HOST',
			$this->api_command.' -a list_objects'
		);
		foreach ($commands as $command) {
			exec($command, $return, $status);
			$this->assertEquals(1, $status, "Command $command returns error code 1");
			$this->ensure_no_php_errors($return);
		}
	}

	/**
	 * 	Test that API doesn't crash when listing objects
	 * Bug #5452
	 */
	public function test_object_listings()
	{
		global $Supported_Object_Types;
		$objtypes = $Supported_Object_Types;
		unset($objtypes[array_search('FILE', $objtypes)]);
		$objtypes[] = 'user';
		foreach ($objtypes as $objtype) {
			exec($this->api_command.' -t '.$objtype.' -a list_objects', $return, $status);
			$this->assertEquals('0', $status, 'Exit with OK status code');
		}
		if ($status != 0)
			print implode("\n", $return);
	}

	/**
	 * Check that api isn't directory dependent
	 * (checks for bug #4409 and similar cases)
	 */
	public function test_from_different_directory()
	{
		global $HOME;
		$curdir = getcwd();
		chdir('/tmp');
		exec("OP5LIBCFG=${HOME}/yml /usr/bin/php $curdir/api/monitor.php  -t timeperiod -o timeperiod_name=API_TIMEPERIOD -o alias=API_TIMEPERIOD", $return, $status);

		$this->assertEquals('0', $status, 'No error when adding timeperiod');
		unset($return);

		exec("OP5LIBCFG=${HOME}/yml /usr/bin/php $curdir/proccess_autoscan.php '*'", $return, $status);
		$this->assertEmpty($return, 'No output when running autoscan');
		$this->assertEquals('0', $status, 'No error when running autoscan');
		chdir($curdir);
	}

	/**
	 * Show changes
	 */
	public function test_show_changes()
	{
		require_once('include/data.inc.php');
		nacoma_db::exec_query('TRUNCATE TABLE changelog');
		exec($this->api_command.' -a show_changes', $return, $status);
		$this->assertEquals('0', $status, 'No error code from show changes');
		$this->assertEquals('No changes', $return[0], "'No changes' printed when there aren't any");
		$return = false;
		exec($this->api_command.' -t timeperiod -o timeperiod_name=API_TIMEPERIOD2 -o alias=API_TIMEPERIOD2', $return, $status);
		$this->assertEquals('0', $status, 'No error code when adding timeperiod');
		$this->assertEmpty($return, 'Nothing printed when adding timeperiod');
		exec($this->api_command.' -a show_changes', $return, $status);
		$this->assertEquals('0', $status, 'No error code from show changes');
		$this->assertTrue(!empty($return), 'Something printed when showing changed timeperiod');

		$return = false;
		exec($this->api_command.' -a undo_config', $return, $status);
		$this->assertEquals('0', $status, 'No error code from undo config');
		$this->assertEmpty($return, 'No output when undoing config');
		exec($this->api_command.' -a show_changes', $return, $status);
		$this->assertEquals('0', $status, 'No error code when examining empty changelog');
		$this->assertEquals('No changes', trim($return[0]), 'Message showing that the changelog is empty is printed');
	}

	/**
	 * We somewhat magically default the username of any non-authed user to
	 * "superuser" - this shouldn't end up in any changelogs if the user did
	 * provide a different username
	 */
	public function test_edit_object_with_username()
	{
		exec($this->api_command.' -t host -o host_name=API_TESTHOST -o template=default-host-template -u test_user', $return, $status);
		$this->assertTrue(is_array($return), 'output');
		$this->assertEquals('0', $status, 'exit code 0');

		$return = false;
		exec($this->api_command.' -a show_changes', $return, $status);
		$return = implode("\n", $return);
		$this->assertEquals('0', $status, 'No error code when examining changelog');
		$this->assertTrue(strpos($return, 'API_TESTHOST') !== false, 'Newly created host should be in changelog: '.$return);
		$this->assertTrue(strpos($return, 'test_user') !== false, 'Username used for creating host is in changelog: '.$return);
		$this->assertTrue(strpos($return, 'superuser') === false, 'Global magical super-username is not in chanelog: '.$return);

		$return = false;
		exec($this->api_command.' -a undo_config', $return, $status);
		$this->assertEquals('0', $status, 'No error code from undo config');
		$this->assertEmpty($return, 'No output when undoing config');
	}

}
