<?php
/**
 * Test higher level functionality in object_nacoma_Model
 * Lower level tests are in oatest.php
 */
class ObjectTest extends PHPUnit_Framework_TestCase
{
	protected $backupGlobalsBlacklist = array('master_ary', 'Supported_Object_Types');

	function setUp()
	{
		evil_global_setup();
		set_user('host_7', array('op5_admins'));
		global $HOME, $IMPORTING;
		$cfgs = array($HOME.'mangleconfig.cfg', $HOME.'testconfig.cfg');
		require_once('include/import.inc.php');
		$this->assertTrue(
			import_config('phptap', $cfgs),
			"import_config('phptap', '" . print_r($cfgs, true) . "')"
		);
		$IMPORTING = false;
	}

	function tearDown()
	{
		evil_global_teardown();
		nacoma_db::exec_query('DELETE FROM management_pack WHERE management_pack_name = "a mgmt pack"');
		op5objstore::instance()->clear();
	}

	function testTemplateDeletion()
	{
		$tmpl = new host_template_nacoma_Model();
		$tmpl->get_default_object();
		$tmpl->obj['name'] = 'tmp_tmpl';
		$tmpl->obj['check_command'] = $tmpl->obj['notification_period'] = 1;
		$this->assertTrue($tmpl->save_object(), "Saving template should work, got ".var_export($tmpl->validation_error_list, true));
		$tmpl_id = $tmpl->id;

		$host = new host_nacoma_Model();
		$host->get_default_object();
		$host->obj['host_name'] = $host->obj['alias'] = $host->obj['address'] = 'tmphost';
		$host->obj['template'] = $tmpl->id;
		$host->save_object();
		$this->assertEmpty($host->validation_error_list, 'Host should be created: '.var_export($host->validation_error_list, true));
		$host_id = $host->id;

		$this->assertTrue(!$tmpl->banish_object(), "Template isn't deleted because it has an active object user");
		$this->assertTrue($host->banish_object(), "Host is deleted");
		$this->assertTrue($tmpl->banish_object(), "Template is deleted when host is gone");

		$tmpl = new host_template_nacoma_Model($tmpl_id);
		$this->assertEmpty($tmpl->obj, 'Template deleted');

		$host = new host_nacoma_Model($host_id);
		unset($host->obj['contacts']);
		$this->assertEmpty($host->obj, 'Host deleted when it\'s template is');
	}

	function testCopyService()
	{
		$srv = new service_nacoma_Model();
		$srv->get_default_object();
		$srv->obj['host_name'] = 1;
		$srv->obj['service_description'] = 'testservice';
		$srv->obj['check_command'] = 1;
		$srv->save_object();
		$this->assertEmpty($srv->validation_error_list, 'Service saved ok');

		$esc = new serviceescalation_nacoma_Model();
		$esc->get_default_object();
		$esc->obj['service'] = $srv->id;
		$esc->save_object();
		$this->assertEmpty($esc->validation_error_list, 'Escalation saved ok');

		$new_srv = $srv->clone_object();
		$new_srv->obj['service_description'] = 'cloned testservice';
		$this->assertEquals($new_srv->clone_id, $srv->id, 'New service has old service\'s id as clone_id');
		$tmp = $srv->find_slave_objects();
		$this->assertEquals($tmp['serviceescalation'][0], $esc->id, 'Old service has the new escalation');
		$this->assertEmpty($new_srv->find_slave_objects(), 'No dependant objects yet');
		$new_srv->save_object();
		$this->assertEmpty($new_srv->validation_error_list, 'New service saved ok');
		$tmp = $srv->find_slave_objects();
		$this->assertEquals($tmp['serviceescalation'][0], $esc->id, 'Old service still has the new escalation');
		$tmp2 = $new_srv->find_slave_objects();
		$this->assertTrue(!empty($tmp2['serviceescalation']), 'New service has its own service escalation');
		$this->assertTrue($tmp2['serviceescalation'][0] != $tmp['serviceescalation'][0], 'Different escalations are assigned to the two services');
	}

	function testServiceNoCommand() {
		# test both that this works and doesn't
		# because I don't want to pass tests because, say, the host name is invalid
		foreach (array(1, false) as $cmd) {
			$srv = new service_nacoma_Model();
			$srv->get_default_object();
			$srv->obj['service_description'] = 'a service';
			$srv->obj['host_name'] = 1;
			$srv->obj['check_command'] = $cmd;
			$this->assertEquals(!!$cmd, $srv->save_object(), sprintf("Save new object with %s command should %swork", ($cmd ? "valid" : "invalid"), ($cmd ? "" : "not ")));
			if ($cmd) {
				$this->assertEmpty($srv->validation_error_list, "Validation error list should be empty after saving worked");
				$this->assertTrue($srv->banish_object(), "Deleting the newly created object should work");
			}
		}
	}

	function groupProvider() {
		return array(
			array('host'),
			array('service'),
			array('contact')
		);
	}

	/**
 	 * @dataProvider groupProvider
	 */
	function testGroup($type) {
		$grouptype = $type . 'group';
		$member = nacoma_db::exec_query('select id from '.$type.' where name is null limit 1')->fetch(PDO::FETCH_NUM);
		$groupmember = nacoma_db::exec_query('select id from '.$grouptype.' limit 1')->fetch(PDO::FETCH_NUM);
		$this->assertInternalType('array', $member, "Tried to fetch a $type object");
		$this->assertInternalType('array', $groupmember, "Tried to fetch a $grouptype object");
		$member = $member[0];
		$groupmember = $groupmember[0];
		$this->assertGreaterThan(0, $member, "Tried to fetch a $type object");
		$this->assertGreaterThan(0, $groupmember, "Tried to fetch a $grouptype object");
		$class = $grouptype.'_nacoma_Model';
		$obj = new $class();
		$obj->get_default_object();
		$obj->obj[$grouptype.'_name'] = 'test '.$grouptype;
		$obj->obj['alias'] = 'test '.$grouptype;
		$this->assertTrue($obj->save_object(), "Tried to save a {$grouptype} with no members, but failed: ".var_export($obj->validation_error_list, true));
		$obj->obj['members'] = array($member);
		$this->assertTrue($obj->save_object(), "Saving should be successful when members are added to the {$grouptype} - got ".var_export($obj->validation_error_list, true));
		$reloaded_obj = new $class($obj->id);
		$this->assertEquals($reloaded_obj->obj['members'], $obj->obj['members'], 'Members are saved correctly');
		$this->assertTrue(!isset($reloaded_obj->obj[$grouptype.'_members']), "No {$grouptype}_members added");
		$obj->obj['members'] = array();
		$obj->obj[$grouptype.'_members'] = array($groupmember);
		$this->assertTrue($obj->save_object(), "Saving is successful when {$grouptype}_members");
		$reloaded_obj = new $class($obj->id);
		$this->assertEquals($reloaded_obj->obj[$grouptype.'_members'], $obj->obj[$grouptype.'_members'], "{$grouptype}_members are saved correctly");
		$this->assertTrue(!isset($reloaded_obj->obj['members']), "No members added");

		$clone = $obj->clone_object();
		$this->assertEquals($clone->obj[$grouptype.'_members'], $obj->obj[$grouptype.'_members'], "Cloned object retains {$grouptype}_members attribute");
		$id = $obj->id;
		$obj->banish_object();
		$new_obj = new $class($id);
		$this->assertEmpty($new_obj->obj, 'Object was really deleted');
	}

	function template_helper($type) {
		$props = $type == 'contact' ? array('address1', 'address2') : array('notes', 'action_url');

		$obj_nacoma_Model = $type.'_nacoma_Model';
		$tmpl_nacoma_Model = $type.'_template_nacoma_Model';
		$gptmpl = new $tmpl_nacoma_Model();
		$gptmpl->obj['name'] = 'Grandparent test template';
		$gptmpl->obj[$props[0]] = "This is a {$props[0]} from grandparent";
		$this->assertTrue($gptmpl->save_object(), "Saving grandparent $type template should work, got " . var_export($gptmpl->validation_error_list, true));
		$ptmpl = new $tmpl_nacoma_Model();
		$ptmpl->obj['name'] = 'Parent test template';
		$ptmpl->obj[$props[1]] = 'http://schwarzenegger.com';
		$ptmpl->obj['template'] = $gptmpl->id;
		$this->assertTrue($ptmpl->save_object(), "Saving parent $type template should work, got " . var_export($ptmpl->validation_error_list, true));
		$obj = new $obj_nacoma_Model();
		$obj->obj['template'] = $ptmpl->id;
		if ($type == 'service') {
			$obj->obj['host_name'] = 'host_9';
			$obj->obj['service_description'] = 'test service';
			$obj->obj['check_interval'] = $obj->obj['retry_interval'] = $obj->obj['check_period'] = $obj->obj['check_command'] = 1;
		} else {
			$obj->obj['alias'] = $obj->obj[$type.'_name'] = 'test '.$type;
		}
		if ($type !== 'contact')
			$obj->obj['max_check_attempts'] = $obj->obj['notification_interval'] = $obj->obj['notification_period'] = 1;
		$this->assertTrue($obj->save_object(), "Saving the $type object should work, got " . var_export($obj->validation_error_list, true));
		$obj = new $obj_nacoma_Model($obj->id);
		$obj->expand_object(true);
		$ptmpl->expand_object(true);
		$gptmpl->expand_object(true);
		$this->assertEquals($obj->obj[$props[1]], $ptmpl->obj[$props[1]], $type . ' action url should be set from direct template');
		$this->assertEquals($obj->obj[$props[0]], $gptmpl->obj[$props[0]], $type . ' notes should be set from indirect template');
		$this->assertEquals($ptmpl->obj[$props[0]], $gptmpl->obj[$props[0]], $type . ' notes should be set to template from parent template');

		$this->assertTrue(
			!$gptmpl->delete_object(),
			'Deleting grandparent should not work, but it did: ' .
				print_r($gptmpl->delete_message, true)
		);
		$this->assertTrue(
			!$ptmpl->delete_object(),
			'Deleting parent should not work, but it did: ' .
				print_r($ptmpl->delete_message, true)
		);
		$this->assertTrue(
			$obj->delete_object(),
			"Deleting object should now work, but it didn't: " .
				print_r($obj->delete_message, true)
		);
		$this->assertTrue(
			$ptmpl->delete_object(),
			"Deleting parent should now work, but it didn't: " .
				print_r($ptmpl->delete_message, true)
		);

		$this->assertTrue(
			$gptmpl->delete_object(),
			"Deleting grandparent should now work, but it didn't: " .
				print_r($gptmpl->delete_message, true)
		);
	}

	function testTemplateTemplate() {
		$this->template_helper('host');
		$this->template_helper('service');
		$this->template_helper('contact');
	}

	function testResaveServiceWithSemicolons() {
		$svc = new service_nacoma_Model('host_9;HTTP Server');
		$svc->save_object();
		$this->assertEmpty($svc->validation_error_list, 'Resave service with escaped ; ok');
		$svc->obj['check_command'] = 'check_http!arg;with;unescaped;semicolons';
		$this->assertTrue(!$svc->save_object(), 'Save service with unescaped ; not ok');
		$svc->obj['check_command'] = 'check_http!arg\;with;semicolons';
		$this->assertTrue(!$svc->save_object(), 'Save service with some unescaped ; not ok');
	}

	function testSaveAndPermissions() {
		$host = new host_nacoma_Model('host_9');
		$svc = new service_nacoma_Model();
		$svc->obj['host_name'] = $host->id;
		$svc->obj['service_description'] = 'testservice';
		$svc->normalize(array('template' => 'default-service'));
		$svc->obj['check_command'] = 1;
		$svc->save_object();
		$svc->prepare_draw();
		$this->assertEmpty($svc->validation_error_list, 'New service saved');
		$fellow_slaves = $svc->get_fellow_slaves();
		$this->assertTrue(count($fellow_slaves) > 1, 'The list of fellow slaves contains objects other than this');
		$old_svc = new service_nacoma_Model('host_9;HTTP Server');
		$this->assertContains($old_svc->get_object_name(), $fellow_slaves, 'The HTTP Server service on this host is a fellow slave');
		$svc->banish_object();
	}

	/**
	 * bug #6488
	 */
	function testPropagateAppend() {
		$host = new host_nacoma_Model('host_9');
		$denom = $host->denormalize();
		$host->propagate_variables(array('hostgroups' => $denom['hostgroups']), array('host_11', 'host_23'), 'append');
		$propagated = new host_nacoma_Model('host_23');
		$this->assertCount(2, $propagated->obj['hostgroups'], 'host_23 should have its original + host_9\'s hostgroup = 2, was '.var_export($propagated->obj['hostgroups'], true));
		$this->assertTrue(!in_array('hostgroup_4', $propagated->obj['hostgroups']), 'host_23 should not get host_11s old hostgroups');
	}

	function testPropagateSubtract() {
		$host = new host_nacoma_Model('host_9');
		$host->propagate_variables(array('hostgroups' => $host->obj['hostgroups']), array('host_11', 'host_23'), 'subtract');
		$propagated = new host_nacoma_Model('host_23');
		$this->assertEquals(count($propagated->obj['hostgroups']), 1, 'host_23 should no longer have host_9\'s hostgroup => 1');
		$this->assertEquals(count($propagated->obj['hostgroups']), 1, 'host_11 should no longer have host_9\'s hostgroup => 1');
	}

	/**
	 * bug #6661
	 */
	function testTemplateCustomVar() {
		$tmpl = new host_template_nacoma_Model('default-host-template');
		$this->assertEquals($tmpl->custom_vars['_random_var'], 'random_value', 'The template should have the expected custom variable');
		$hst1 = new host_nacoma_Model('host_1');
		$this->assertEquals($hst1->custom_vars['_random_var'], 'not_a_random_value', 'The object that overrides template should have its custom variable');
		$hst2 = new host_nacoma_Model('host_2');
		$this->assertEmpty($hst2->custom_vars, "The object that doesn't override template shouldn't have a custom variable");
		$hst2->expand_object();
		$this->assertEquals($hst2->custom_vars['_random_var'], 'random_value', "The object that doesn't override template should load templates custom variable when asked");
		$this->assertTrue($hst1->save_object(), 'Saving unmodified object should work');
		$this->assertTrue($hst2->save_object(), 'Saving unmodified object should work');
		$tmpl->custom_vars['_random_var'] = 'new value';
		$this->assertTrue($tmpl->save_object(), 'Saving template with new custom var should work, got' . var_export($tmpl->validation_error_list, true));
		$tmpl = new host_template_nacoma_Model('default-host-template');
		$this->assertEquals($tmpl->custom_vars['_random_var'], 'new value', 'The template should have the expected custom variable');
		$hst1 = new host_nacoma_Model('host_1');
		$this->assertEquals($hst1->custom_vars['_random_var'], 'not_a_random_value', 'The object that overrides template should still have its custom variable');
		$hst2 = new host_nacoma_Model('host_2');
		$this->assertEmpty($hst2->custom_vars, 'The object that doesn\'t override template should still not have a custom variable');
		$hst2->expand_object();
		$this->assertEquals($hst2->custom_vars['_random_var'], 'new value', 'The object that doesn\'t override template should still load templates custom variable when told');
	}

	function testUnicode() {
		$strings = array(
			'今年から貝が胃に棲み始めました。' => true, #kanji - with scary full stop
			'আমার হভারক্রাফ্ট ইল-এ ভর্তি হয়ে' => true, #bengal - scary all over, with "vowel stop" and nastiness
			'  ' => true, #klingon - private plane
			"Looks alright (but isn't)" => false, #parenthesis
			"Now we backspace\x08" => false, #backspace character
		);

		$host = new host_nacoma_Model('new host');
		$host->get_default_object();
		foreach ($strings as $string => $legal) {
			$host->obj['host_name'] = $string;
			$this->assertTrue($legal == $host->save_object(), "Setting the identifier of a host to '$string' should" . ($legal ? " be allowed, but wasn't" : "n't be allowed, but was") . ": " . var_export($host->validation_error_list, true));
		}

		$service = new service_nacoma_Model();
		$service->normalize(array('template' => 'default-service'));
		$service->obj['check_command'] = 1;
		$service->obj['host_name'] = $host->id;
		foreach ($strings as $string => $legal) {
			$service->obj['service_description'] = $string;
			$this->assertTrue($legal == $service->save_object(), "Setting the identifier of a service to '$string' should" . ($legal ? " be allowed, but wasn't" : "n't be allowed, but was") . ": " . var_export($service->validation_error_list, true));
		}
	}

	function testContactWithUser() {
		$contact = new contact_nacoma_Model();
		$contact->obj['enable_access'] = 1;
		$contact->obj['alias'] = $contact->obj['contact_name'] = 'test_contact_with_user';
		$contact->obj['password'] = 'test_password';
		$contact->obj['groups'] = array('contact_limited');
		$this->assertTrue($contact->save_object(), "Saving contact with test user should work, got " . var_export($contact->validation_error_list, true));
		$user = new user_nacoma_Model('test_contact_with_user');
		$this->assertEquals($user->obj['groups'], $contact->obj['groups'], "Group list for loaded user should be equal to group list for contact-with-user");
	}

	function testDeleteNumericUser() {
		$user = new user_nacoma_Model();
		$user->obj['username'] = '0123456789';
		$user->obj['realname'] = 'numeric';
		$user->obj['password'] = 'test_password';
		$user->obj['groups'] = array('contact_limited');
		$this->assertTrue($user->save_object(), "Saving user with numeric name should work, got " . var_export($user->validation_error_list, true));
		user_nacoma_Model::drop_cache();
		$user = new user_nacoma_Model('0123456789');
		$this->assertEquals($user->obj['username'], '0123456789', 'The username should still have the leading zero');
		$this->assertEquals($user->obj['groups'], array('contact_limited'), "Group list for loaded user should be equal to group list for contact-with-user");
		$this->assertTrue($user->banish_object(), "Banishing the newly created numeric user should work");
		$user = new user_nacoma_Model('0123456789');
		$this->assertEmpty($user->obj, "Banished numeric user should be gone");
	}

	/* we broke this when password_algo was missing */
	function testDenormalizeUser()
	{
		$user = new user_nacoma_Model('administrator');
		$user->save_object();
	}

	function testManagementPack() {
		$mp = new management_pack_nacoma_Model();
		$mp->obj['hostgroup'] = 'hostgroup_1';
		$mp->obj['management_pack_name'] = 'the management pack';
		$mp->obj['description'] = 'a management pack';
		$mp->obj['icon'] = 'edit.png';
		$this->assertTrue($mp->save_object(), 'Saving a new management pack should be OK');
		$this->assertEmpty($mp->validation_error_list, 'There should be no validation errors when saving new pack');
		$old_mp = new management_pack_nacoma_Model('the management pack');
		$this->assertTrue(!empty($old_mp->obj), 'Loading a saved management pack should work');
		$this->assertTrue(is_numeric($old_mp->id), 'There should be a numeric id for a saved management pack');
		$this->assertTrue($old_mp->id > 0, 'The saved management pack should have an ID greater than 0');
		$this->assertEquals($old_mp->obj['management_pack_name'], 'the management pack', 'The saved name should be loaded');
		$this->assertEquals($old_mp->obj['hostgroup'], 'hostgroup_1', "The original, non-normalized hostgroup name should be loaded again");
		$old_mp->obj['management_pack_name'] = 'new name';
		$old_mp->custom_vars['_DBNAME'] = 'This names a database';
		$this->assertTrue($old_mp->save_object(), 'Renaming the changed management pack should work');
		$new_mp = new management_pack_nacoma_Model('new name');
		$this->assertTrue(!empty($new_mp->obj), 'Loading a edited management pack should work');
		$this->assertTrue(is_numeric($new_mp->id), 'There should be a numeric id for a edited management pack');
		$this->assertTrue($new_mp->id > 0, 'The edited management pack should have an ID greater than 0');
		$this->assertEquals($new_mp->obj['management_pack_name'], 'new name', 'The edited name should be loaded');
		$this->assertEquals($new_mp->custom_vars['_DBNAME'], 'This names a database', 'The saved custom var should be loaded');
		$this->assertTrue($new_mp->banish_object(), 'Deleting the management pack should work');
		$this->assertTrue(
			is_array($new_mp->delete_message),
			"When deleting the management pack, there shouldn't be an error " .
				"message, was " . print_r($new_mp->delete_message, true)
		);
	}

	function testRenameHostgroupWithMgmtPack() {
		$hg = new hostgroup_nacoma_Model();
		$hg->obj['hostgroup_name'] = 'hostgroup before rename';
		$this->assertTrue($hg->save_object(), 'Saving new hostgroup should work');
		$this->assertEmpty($hg->validation_error_list, 'The validation error list for new hostgroup should be empty');

		$mgmt = new management_pack_nacoma_Model();
		$mgmt->obj['management_pack_name'] = 'a mgmt pack';
		$mgmt->obj['hostgroup'] = $hg->obj['hostgroup_name'];
		$mgmt->obj['description'] = 'a management pack';
		$mgmt->obj['icon'] = 'edit.png';
		$this->assertTrue($mgmt->save_object(), 'Saving new mgmt pack should work, got '.var_export($mgmt->validation_error_list, true));
		$this->assertEmpty($mgmt->validation_error_list, 'The validation error list for new mgmt pack should be empty');

		$hg2 = new hostgroup_nacoma_Model($hg->id);
		$this->assertEquals($hg2->obj['hostgroup_name'], $hg->obj['hostgroup_name'], 'The hostgroup should load correctly');
		$hg2->obj['hostgroup_name'] = 'hostgroup after rename';
		$this->assertTrue($hg2->save_object(), 'Saving renamed hostgroup should work');
		$this->assertEmpty($hg2->validation_error_list, 'The validation error list for renamed hostgroup should be empty');

		$mgmt2 = new management_pack_nacoma_Model($mgmt->id);
		$this->assertEquals($mgmt2->obj['management_pack_name'], $mgmt->obj['management_pack_name'], 'The management pack should load correctly');
		$this->assertEquals($mgmt2->obj['hostgroup'], $hg2->obj['hostgroup_name'], 'The hostgroup parameter for the management pack should be renamed when the group is');

		$this->assertTrue($mgmt2->banish_object(), 'Banish object for management pack should work');
		$this->assertTrue($hg2->banish_object(), 'Banish object for hostgroup should work');
	}

	/**
	 * if the mgmtpack points to a nonexisting hostgroup, we cannot die
	 */
	function testBrokenManagementPack() {
		$hg = new hostgroup_nacoma_Model();
		$hg->obj['hostgroup_name'] = 'hostgroup name';
		$this->assertTrue($hg->save_object(), 'Saving new hostgroup should work');
		$this->assertEmpty($hg->validation_error_list, 'The validation error list for new hostgroup should be empty');

		$mgmt = new management_pack_nacoma_Model();
		$mgmt->obj['management_pack_name'] = 'a mgmt pack';
		$mgmt->obj['hostgroup'] = $hg->obj['hostgroup_name'];
		$mgmt->obj['description'] = 'a management pack';
		$mgmt->obj['icon'] = 'edit.png';
		$this->assertTrue($mgmt->save_object(), 'Saving new mgmt pack should work, got '.var_export($mgmt->validation_error_list, true));

		$mgmt = new management_pack_nacoma_Model('a mgmt pack');
		nacoma_db::exec_query("UPDATE management_pack SET hostgroup = 'not a hostgroup name' WHERE id = '{$mgmt->id}'");

		$tbl = management_pack_nacoma_Model::get_objects_for_user();
		$this->assertTrue(!in_array('a mgmt pack', $tbl), "The managment pack should be inaccessible when the hostgroup is broken");
		$mgmt = new management_pack_nacoma_Model('a mgmt pack');
		$this->assertGreaterThan(0, $mgmt->id, "The managment pack should be found when explicitly loading it when the hostgroup is broken");
		$this->assertTrue($mgmt->save_object(), 'Saving broken mgmt pack should not work, except it does, and apparently only normalize() validates and validate_object() just goofs around for a bit, and we\'re close to release, and srsly nacoma? '.var_export($mgmt->validation_error_list, true));

		$mgmt->obj['hostgroup'] = $hg->obj['hostgroup_name'];
		$this->assertTrue($mgmt->save_object(), 'Saving fixed mgmt pack should work, got '.var_export($mgmt->validation_error_list, true));

		$this->assertTrue($mgmt->banish_object(), 'Banish object for management pack should work');
		$this->assertTrue($hg->banish_object(), 'Banish object for hostgroup should work');
	}

	function testRecursiveTemplate() {
		$obj = new host_nacoma_Model('host_defined_by_recursive_template');
		$obj->expand_object(true);
		$this->assertEquals('foo', $obj->obj['alias'], "The alias should be inherited as 'foo'");
		$obj->collapse_object(true);
		$this->assertTrue(!isset($obj->obj['alias']), "The alias should have been compacted away");
	}

	# These pnp xml files exist: host17 (host itself, Bandwidth, Mem Usage), host14 (host itself, PING)

	function testGraphCollections() {
		$obj = new graph_collection_nacoma_Model('test_invalid');
		$this->assertEmpty($obj->obj, 'Loading invalid collection should return empty object');
		$this->assertTrue(!$obj->id, 'Loading invalid collection should return false id');

		$obj = new graph_collection_nacoma_Model();
		$obj->obj['page_name'] = 'test1';
		$obj->obj['services'] = array('host17', 'host17;Mem Usage');
		$obj->obj['use_regex'] = false;
		$this->assertTrue($obj->save_object(), 'Saving collection should work, but got '.var_export($obj->validation_error_list, true));
		$this->assertTrue($obj->save_object(), 'Saving same collection twice should work, but got '.var_export($obj->validation_error_list, true));
		$loaded_obj = new graph_collection_nacoma_Model('test1');
		$this->assertEquals($loaded_obj->obj, $obj->obj, sprintf('Loaded object should be identical to what we saved, got %s vs %s', var_export($loaded_obj->obj, true), var_export($obj->obj, true)));

		$obj = new graph_collection_nacoma_Model('test1');
		$obj->obj['services'] = array_merge($obj->obj['services'], array('host14', 'host14;PING'));
		$obj->obj['use_regex'] = false;
		$this->assertTrue($obj->save_object(), 'Resaving collection with more services should work, but got '.var_export($obj->validation_error_list, true));
		$loaded_obj = new graph_collection_nacoma_Model('test1');
		$this->assertEquals($loaded_obj->obj, $obj->obj, sprintf('Re-loaded object should be identical to what we saved, got %s vs %s', var_export($loaded_obj->obj, true), var_export($obj->obj, true)));

		$obj = new graph_collection_nacoma_Model('test1');
		$obj->obj['page_name'] = 'test2';
		$this->assertTrue($obj->save_object(), 'Saving collection with new name should work, but got '.var_export($obj->validation_error_list, true));
		$loaded_obj = new graph_collection_nacoma_Model('test2');
		$this->assertEquals($loaded_obj->obj, $obj->obj, sprintf('Loading renamed object should become identical to what we saved, got %s vs %s', var_export($loaded_obj->obj, true), var_export($obj->obj, true)));

		$obj = new graph_collection_nacoma_Model('test1');
		$this->assertEmpty($obj->obj, 'Loading old name for renamed collection should return empty object');
		$this->assertTrue(!$obj->id, 'Loading old name for renamed collection should return false id');

		$obj = new graph_collection_nacoma_Model('test2');
		$this->assertTrue($obj->delete_object(), 'Object deletion should work');
		$obj = new graph_collection_nacoma_Model('test2');
		$this->assertEmpty($obj->obj, 'Loading deleted collection should return empty object');
		$this->assertTrue(!$obj->id, 'Loading deleted collection should return false id');
	}

	function testCombinedGraph() {
		$obj = new combined_graph_nacoma_Model('test_invalid');
		$this->assertEmpty($obj->obj, 'Loading invalid combined graph should return empty object');
		$this->assertTrue(!$obj->id, 'Loading invalid combined graph should return false id');

		$obj = new combined_graph_nacoma_Model();
		$obj->obj['title'] = 'test1';
		$obj->obj['comment'] = 'yes';
		$obj->obj['type'] = 'line2';
		$obj->obj['services'] = array('host17', 'host17;Mem Usage');
		$this->assertTrue($obj->save_object(), 'Saving combined graph should work, but got '.var_export($obj->validation_error_list, true));
		$this->assertTrue($obj->save_object(), 'Saving same combined graph twice should work, but got '.var_export($obj->validation_error_list, true));
		$loaded_obj = new combined_graph_nacoma_Model('test1');
		$this->assertEquals($loaded_obj->obj, $obj->obj, sprintf('Loaded object should be identical to what we saved, got %s vs %s', var_export($loaded_obj->obj, true), var_export($obj->obj, true)));

		$obj = new combined_graph_nacoma_Model('test1');
		$obj->obj['services'] = array_merge($obj->obj['services'], array('host14', 'host14;PING'));
		$this->assertTrue($obj->save_object(), 'Resaving combined graph with more services should work, but got '.var_export($obj->validation_error_list, true));
		$loaded_obj = new combined_graph_nacoma_Model('test1');
		$this->assertEquals($loaded_obj->obj, $obj->obj, sprintf('Re-loaded object should be identical to what we saved, got %s vs %s', var_export($loaded_obj->obj, true), var_export($obj->obj, true)));

		$obj = new combined_graph_nacoma_Model('test1');
		$obj->obj['title'] = 'test2';
		$this->assertTrue($obj->save_object(), 'Saving combined graph with new name should work, but got '.var_export($obj->validation_error_list, true));
		$loaded_obj = new combined_graph_nacoma_Model('test2');
		$this->assertEquals($loaded_obj->obj, $obj->obj, sprintf('Loading renamed object should become identical to what we saved, got %s vs %s', var_export($loaded_obj->obj, true), var_export($obj->obj, true)));

		$obj = new combined_graph_nacoma_Model('test1');
		$this->assertEmpty($obj->obj, 'Loading old name for renamed combined graph should return empty object');
		$this->assertTrue(!$obj->id, 'Loading old name for renamed combined graph should return false id');

		$obj = new combined_graph_nacoma_Model('test2');
		$this->assertTrue($obj->delete_object(), 'Object deletion should work');
		$obj = new combined_graph_nacoma_Model('test2');
		$this->assertEmpty($obj->obj, 'Loading deleted combined graph should return empty object');
		$this->assertTrue(!$obj->id, 'Loading deleted combined graph should return false id');
	}

	/*
	function testGraphTemplate() {
		$obj = new graph_template_nacoma_Model('test_invalid');
		$this->assertEmpty($obj->obj, 'Loading invalid graph template should return empty object');
		$this->assertTrue(!$obj->id, 'Loading invalid graph template should return false id');

		$obj = new graph_template_nacoma_Model();
		$obj->normalize(array('check' => 'check-host-alive'));
		$obj->obj['graph_template'] = 'templates.dist/check_nt_mem.php';
		$this->assertTrue($obj->save_object(), 'Saving graph template should work, but got '.var_export($obj->validation_error_list, true));
		$this->assertTrue($obj->save_object(), 'Saving same graph template twice should work, but got '.var_export($obj->validation_error_list, true));
		$loaded_obj = new graph_template_nacoma_Model('check-host-alive');
		$this->assertEquals($loaded_obj->denormalize(), $obj->denormalize(), sprintf('Loaded object should be identical to what we saved, got %s vs %s', var_export($loaded_obj->obj, true), var_export($obj->obj, true)));

		$obj = new graph_template_nacoma_Model('check-host-alive');
		$obj->obj['graph_template'] = 'templates.dist/check_nwstat_LOAD1.php';
		$this->assertTrue($obj->save_object(), 'Resaving graph template with different template should work, but got '.var_export($obj->validation_error_list, true));
		$loaded_obj = new graph_template_nacoma_Model('check-host-alive');
		$this->assertEquals($loaded_obj->denormalize(), $obj->denormalize(), sprintf('Re-loaded object should be identical to what we saved, got %s vs %s', var_export($loaded_obj->obj, true), var_export($obj->obj, true)));

		$obj = new graph_template_nacoma_Model('check-host-alive');
		$obj->normalize(array('check' => 'check_ssh'));
		$this->assertTrue($obj->save_object(), 'Saving graph template with new name should work, but got '.var_export($obj->validation_error_list, true));
		$loaded_obj = new graph_template_nacoma_Model('check_ssh');
		$this->assertEquals($loaded_obj->denormalize(), $obj->denormalize(), sprintf('Loading renamed object should become identical to what we saved, got %s vs %s', var_export($loaded_obj->obj, true), var_export($obj->obj, true)));

		$obj = new graph_template_nacoma_Model('check-host-alive');
		$this->assertEmpty($obj->obj, 'Loading old name for renamed graph template should return empty object');
		$this->assertTrue(!$obj->id, 'Loading old name for renamed graph template should return false id');

		$obj = new graph_template_nacoma_Model('check_ssh');
		$this->assertTrue($obj->delete_object(), 'Object deletion should work, but got: '.$obj->delete_message);
		$obj = new graph_template_nacoma_Model('check_ssh');
		$this->assertEmpty($obj->obj, 'Loading deleted graph template should return empty object');
		$this->assertTrue(!$obj->id, 'Loading deleted graph template should return false id');
	}*/
}
