<?php
require_once('devtools/tapunit.php');
if (empty($main_tap))
	$main_tap = new phptap();

/**
 * Tests for the "undo my changes" functionality
 */
class RevertTest extends TapUnit
{
	function setUp()
	{
		revert_users_changes();
		nacoma_db::exec_query('DELETE FROM changelog');
	}

	function testRevertUserChanges()
	{
		set_user('host_7', array('op5_admins'));

		// Set proper access where user is only contact.
		set_user('contact_limited', array('contact_limited'));
		$orig_obj = new host_nacoma_Model('host_10');
		$obj = new host_nacoma_Model('host_10');
		$obj->obj['alias'] = 'foobar';
		$obj->save_object();
		$this->ok_empty($obj->validation_error_list, 'User can save host');
		$obj = new host_nacoma_Model('host_10');
		$this->ok_eq($obj->obj['alias'], 'foobar', "User can modify host");
		revert_users_changes();
		$obj = new host_nacoma_Model('host_10');
		$this->ok_eq($obj->obj['alias'], $orig_obj->obj['alias'], "User's changes are reset when they should");
		$entries = nacoma_db::exec_query('SELECT COUNT(1) FROM changelog');
		$count = $entries->fetchAll(PDO::FETCH_NUM);
		$this->ok_eq($count, array(array(0)), "Changelog is emptied when reverting changes");
	}

	function testRevertDeletesMyNewHostdep()
	{
		set_user('host_7', array('op5_admins'));

		$obj = new hostdependency_nacoma_Model();
		$obj->get_default_object();
		$obj->id = 'new';
		$host = new host_nacoma_Model('host_10');
		$obj->obj['host_name'] = $host->id;
		$host = new host_nacoma_Model('host_11');
		$obj->obj['dependent_host_name'] = $host->id;
		$obj->obj['notification_failure_criteria'] = array('d','u');
		$obj->save_object();
		unset($entries);
		$entries = nacoma_db::exec_query('SELECT COUNT(1) FROM hostdependency WHERE dependent_host_name='.nacoma_db::quote_string($host->id));
		$count = $entries->fetchAll(PDO::FETCH_NUM);
		$this->ok_eq($count, array(array(1)), "Hostdependency is created");
		revert_users_changes();
		unset($entries);
		$entries = nacoma_db::exec_query('SELECT COUNT(1) FROM hostdependency WHERE dependent_host_name='.nacoma_db::quote_string($host->id));
		$count = $entries->fetchAll(PDO::FETCH_NUM);
		$this->ok_eq($count, array(array(0)), "Hostdependency is deleted after revert");
	}

	function testRevertDeletesMyNewHost()
	{
		$obj = new host_nacoma_Model();
		$obj->get_default_object();
		$obj->obj['host_name'] = 'temp object2';
		$obj->obj['alias'] = 'temp object';
		$obj->obj['address'] = 'no address';
		$obj->id = 'new';
		$obj->save_object();
		$this->ok_empty($obj->validation_error_list, 'User can create new host');

		$obj = new host_nacoma_Model('temp object2');
		$this->ok_eq($obj->obj['host_name'], 'temp object2', "New host is saved");
		revert_users_changes();
		$obj = new host_nacoma_Model('temp object2');
		$this->ok_empty($obj->obj, "New host is deleted after revert");
	}

	function testRevertBackToOtherUsersUnsavedState()
	{
		set_user('host_7', array('op5_admins'));
		$obj = new host_nacoma_Model('host_10');
		$obj->obj['alias'] = 'host_7 set alias';
		$obj->obj['notes'] = 'host_7 set note';
		$obj->save_object();
		$this->ok_empty($obj->validation_error_list, 'Admin can save host');

		$obj = new host_nacoma_Model('host_11');
		$obj->obj['alias'] = 'host_7 set alias';
		$obj->save_object();
		$this->ok_empty($obj->validation_error_list, 'Admin can save host');

		$obj = new service_nacoma_Model(181);
		$obj->obj['display_name'] = 'host_7 set display_name';
		$obj->save_object();
		$this->ok_empty($obj->validation_error_list, 'Admin can save service');

		set_user('contact_limited', array('contact_limited'));
		$obj = new host_nacoma_Model('host_10');
		$this->ok_eq($obj->obj['alias'], 'host_7 set alias', 'Admin set alias sticks');
		$this->ok_eq($obj->obj['notes'], 'host_7 set note', 'Admin set note sticks');
		$obj->obj['alias'] = 'overridden alias';
		$obj->save_object();
		$this->ok_empty($obj->validation_error_list, 'User can save host');

		$obj = new host_nacoma_Model('host_10');
		$this->ok_eq($obj->obj['alias'], 'overridden alias', 'Admin set alias sticks');

		revert_users_changes();
		$obj = new host_nacoma_Model('host_10');
		$this->ok_eq($obj->obj['alias'], 'host_7 set alias', "Admin set alias is back after user revert");
		$this->ok_eq($obj->obj['notes'], 'host_7 set note', "Admin set note didn't revert in user revert");

		$obj = new host_nacoma_Model('host_11');
		$this->ok_eq($obj->obj['alias'], 'host_7 set alias', "Admin set alias to unrelated object didn't change in user revert");
		$obj = new service_nacoma_Model(181);
		$this->ok_eq($obj->obj['display_name'], 'host_7 set display_name', "Admin set service display_name to unrelated object didn't change in user revert");
	}

	function testRevertWhenOtherChangesOnTop()
	{

		$obj = new host_nacoma_Model('host_11');
		$orig_alias = $obj->obj['alias'];
		$obj->obj['alias'] = 'contact_limited set alias';
		$obj->save_object();
		$this->ok_empty($obj->validation_error_list, 'User can save host');

		set_user('host_7', array('op5_admins'));

		$obj = new host_nacoma_Model('host_11');
		$this->ok_eq($obj->obj['alias'], 'contact_limited set alias', "User set alias sticks");
		$obj->obj['alias'] = 'overridden alias';
		$obj->save_object();
		$this->ok_empty($obj->validation_error_list, 'Admin can save host');

		set_user('contact_limited', array('contact_limited'));
		$obj = new host_nacoma_Model('host_11');
		$this->ok_eq($obj->obj['alias'], 'overridden alias', "Admin set alias sticks");
		revert_users_changes();
		$obj = new host_nacoma_Model('host_11');
		$this->ok_eq($obj->obj['alias'], 'overridden alias', "Admin set alias isn't reverted after user reverted his previous changes to it");

		$res = nacoma_db::exec_query('SELECT * FROM changelog');
		$ary = $res->fetchAll(PDO::FETCH_ASSOC);
		$this->ok_eq(count($ary), 1, 'Only one entry in the changelog after reverting');
		$oldstate = unserialize($ary[0]['oldstate']);
		$this->ok_eq($oldstate['alias'], $orig_alias, 'The revert causes oldstate to be properly manipulated');
	}

	function testDeleteThenCreate()
	{
		set_user('host_7', array('op5_admins'));

		$obj = new host_nacoma_Model();
		$obj->get_default_object();
		$obj->obj['host_name'] = 'temp object';
		$obj->obj['alias'] = 'temp object';
		$obj->obj['address'] = 'no address';
		$obj->id = 'new';
		$obj->save_object();
		$this->ok_empty($obj->validation_error_list, 'User can save new host');
		$obj->banish_object();
		$this->ok_empty($obj->validation_error_list, 'User can delete new host');
		$obj = new host_nacoma_Model();
		$obj->get_default_object();
		$obj->obj['host_name'] = 'temp object';
		$obj->obj['alias'] = 'temp object';
		$obj->obj['address'] = 'no address';
		$obj->id = 'new';
		$obj->save_object();
		$this->ok_empty($obj->validation_error_list, 'User can save new host');
		revert_users_changes();
		$this->pass("Called revert_users_changes with a new object sharing ID with an old deleted object without crashes");
		$obj->banish_object();
	}

	# Tests for (parts of) bug #4580: revert objects with custom variables
	function testRevertCustom()
	{
		set_user('host_7', array('op5_admins'));
		$obj = new timeperiod_nacoma_Model();
		$obj->get_default_object();
		$obj->obj['timeperiod_name'] = 'temp period';
		$obj->obj['alias'] = 'temp period';
		$obj->custom_vars['december 24 - 26'] = '00:00-24:00';
		$obj->save_object();
		$this->ok_empty($obj->validation_error_list, 'User can save new timeperiod');
		nacoma_db::exec_query('DELETE FROM changelog'); // forget about the creation
		$obj->custom_vars['december 12 - 14'] = '00:00-24:00';
		$obj->save_object();
		$this->ok_empty($obj->validation_error_list, 'User can set custom exception');
		$nobj = new timeperiod_nacoma_Model($obj->id);
		$this->ok_id($nobj->obj['timeperiod_name'], $obj->obj['timeperiod_name'], 'Object is the same after db load');
		$this->ok(!empty($nobj->custom_vars), 'Custom vars set after load');
		$old_custom = $nobj->custom_vars;
		sort($old_custom);
		$new_custom = $obj->custom_vars;
		sort($new_custom);
		$this->ok_id($old_custom, $new_custom, 'Custom vars are identical in db');
		revert_users_changes();
		$nobj = new timeperiod_nacoma_Model($obj->id);
		$this->ok_id($nobj->obj['timeperiod_name'], $obj->obj['timeperiod_name'], 'Object is the same after revert');
		$this->ok(!empty($nobj->custom_vars), 'Custom vars set after revert');
		$this->ok(!isset($nobj->custom_vars['december 12 - 14']), 'New custom var is not set after revert');
		$this->ok(isset($nobj->custom_vars['december 24 - 26']), 'Old custom var is set after revert');
		$this->ok_eq($nobj->custom_vars['december 24 - 26'], '00:00-24:00', 'Old custom var is the same after revert');

		$obj = new timeperiod_nacoma_Model($obj->id); // reload to post-revert state
		unset($obj->custom_vars['december 24 - 26']);
		$obj->save_object();
		$nobj = new timeperiod_nacoma_Model($obj->id);
		$this->ok_empty($nobj->custom_vars, 'Custom vars is empty');
		revert_users_changes();
		$nobj = new timeperiod_nacoma_Model($obj->id);
		$this->ok(isset($nobj->custom_vars['december 24 - 26']), 'Custom var recreated on undo');
		$this->ok_eq($nobj->custom_vars['december 24 - 26'], '00:00-24:00', 'Custom var has correct value');

		$obj = new timeperiod_nacoma_Model($obj->id); // reload to post-revert state
		$obj->custom_vars['december 12 - 14'] = '00:00-24:00';
		$obj->save_object();
		nacoma_db::exec_query('DELETE FROM changelog'); // forget about the creation
		unset($obj->custom_vars['december 12 - 14']);
		$obj->save_object();
		$nobj = new timeperiod_nacoma_Model($obj->id);
		$this->ok_id($nobj->custom_vars, $obj->custom_vars, 'New custom var saved');
		revert_users_changes();
		$nobj = new timeperiod_nacoma_Model($obj->id);
		$this->ok_eq(count($nobj->custom_vars), 2, 'There are two custom vars after revert');

		$obj->banish_object();
	}

	function testRevertM2M()
	{
		$hostgroup_list = array(1,2,3);
		$obj = new host_nacoma_Model();
		$obj->get_default_object();
		$obj->obj['host_name'] = 'temp object';
		$obj->obj['alias'] = 'temp object';
		$obj->obj['address'] = 'no address';
		$obj->obj['hostgroups'] = $hostgroup_list;
		$obj->id = 'new';
		$obj->save_object();
		$this->ok_empty($obj->validation_error_list, "New host created");
		nacoma_db::exec_query('DELETE FROM changelog'); // make object permanent
		$obj->obj['hostgroups'] = array(1,2);
		$obj->save_object();
		$this->ok_empty($obj->validation_error_list, "Hostgroup removed");
		revert_users_changes();
		$new_obj = new host_nacoma_Model($obj->id);
		$this->ok_eq(sort($new_obj->obj['hostgroups']), $hostgroup_list, "Original hostgroup list restored");
		$new_obj->banish_object();
		nacoma_db::exec_query('DELETE FROM changelog'); // make deletion permanent
	}

	function testRevertWithEmpty()
	{
		set_user('host_7', array('op5_admins'));
		$obj = new host_nacoma_Model('host_10');
		$this->ok($obj->id && $obj->id != 'new', 'Loaded object exists');
		$orig_alias = $obj->obj['alias'];
		$obj->obj['alias'] = 'new_test_alias';
		$obj->save_object();
		$this->ok_empty($obj->validation_error_list, 'Alias changed');
		set_user('contact_limited', array('contact_limited'));
		$obj->obj['alias'] = $orig_alias;
		$obj->save_object();
		$this->ok_empty($obj->validation_error_list, 'Alias restored');
		$res = nacoma_db::exec_query('SELECT * FROM changelog');
		$ary = $res->fetchAll(PDO::FETCH_ASSOC);
		$this->ok_eq(count($ary), 2, 'Two changes in changelog before revert');
		set_user('host_7', array('op5_admins'));
		revert_users_changes();
		unset($res);
		$res = nacoma_db::exec_query('SELECT * FROM changelog');
		$ary = $res->fetchAll(PDO::FETCH_ASSOC);
		$this->ok_empty($ary, 'Changelog emptied after change becomes a noop');
		$obj_post = new host_nacoma_Model('host_10');
		$this->ok_eq($obj->obj, $obj_post->obj, 'After revert, the two objects are the same');
		unset($res);
	}
}

runUnit('RevertTest', $main_tap);
